from dataclasses import dataclass
from typing import Iterable, Iterator

from azure.mgmt.loganalytics import LogAnalyticsManagementClient
from azure.mgmt.loganalytics.models import Table, Workspace as NativeWorkspace
from azure.mgmt.loganalytics.operations import TablesOperations

from davidkhala.azure import TokenCredential
from davidkhala.azure.monitor import AbstractResource


class AnalyticsWorkspace:
    def __init__(self, credential: TokenCredential, subscription_id: str):
        self.client = LogAnalyticsManagementClient(credential, subscription_id)

    def list(self) -> Iterable[NativeWorkspace]:
        return self.client.workspaces.list()

    @dataclass
    class Resource(AbstractResource):

        def __init__(self, operations: TablesOperations):
            super().__init__(*[None] * 5)
            self.operations = operations

        def from_resource(self, resource: NativeWorkspace):
            super().from_resource(resource)
            self.immutable_id = resource.customer_id
            return self

    def create(self, resource_group_name, name) -> Resource:
        promise = self.client.workspaces.begin_create_or_update(resource_group_name, name)
        return AnalyticsWorkspace.Resource(self.tables).from_resource(promise.result())

    @property
    def tables(self) -> TablesOperations:
        return self.client.tables

    def delete(self, resource_group_name: str, name: str, force=True):
        """
        :param name:
        :param resource_group_name:
        :param force: Deletes the workspace without the recovery option (kept for 14 days).
        """
        promise = self.client.workspaces.begin_delete(resource_group_name, name, force)
        promise.result()

    def get(self, resource_group_name, name) -> Resource:
        resource = self.client.workspaces.get(resource_group_name, name)
        return AnalyticsWorkspace.Resource(self.tables).from_resource(resource)


class AnalyticsTable:
    def __init__(self, workspace: AnalyticsWorkspace.Resource):
        self.tables = workspace.operations
        self.workspace = workspace.name
        self.resource_group_name = workspace.resource_group_name

    def list(self, *, no_system: bool = False) -> Iterator[Table]:
        for table in self.tables.list_by_workspace(self.resource_group_name, self.workspace):
            if no_system and not str(table.name).endswith('_CL'):
                continue
            yield table

    def create(self, table_name: str) -> Table:
        parameters = Table()
        promise = self.tables.begin_create_or_update(self.resource_group_name, self.workspace, table_name, parameters)
        return promise.result()



import os

from azure.core.credentials import TokenCredential
from azure.identity import DefaultAzureCredential

from davidkhala.azure.auth import from_service_principal, default


def credentials() -> TokenCredential | DefaultAzureCredential:
    client_secret = os.environ.get('CLIENT_SECRET')
    if client_secret:
        return from_service_principal(
            tenant_id=os.environ.get('TENANT_ID'),
            client_id=os.environ.get('CLIENT_ID'),
            client_secret=client_secret,
        )
    return default()



from azure.ai.agents.models import Agent
from azure.ai.projects import AIProjectClient
from azure.ai.projects.models import PromptAgentDefinition, AgentDetails
from openai import OpenAI
from openai.types.responses.response import Response

from davidkhala.azure import TokenCredential


class Project:
    def __init__(self, foundry_id, project, credential: TokenCredential):
        self.client = AIProjectClient(
            endpoint=f"https://{foundry_id}.services.ai.azure.com/api/projects/{project}",
            credential=credential,
        )
        self.agent: Agent | None = None
        self.model: str | None = None

    def as_chat(self, model: str, sys_prompt: str = None, *, agent_name: str):
        if agent_name:
            self.agent = self.client.agents.create_version(
                agent_name=agent_name,
                definition=PromptAgentDefinition(
                    model=model,
                    instructions=sys_prompt
                ),
            )
        else:
            self.model = model

    @property
    def agents(self) -> list[AgentDetails]:
        return list(self.client.agents.list())

    def chat(self, *user_prompt, **kwargs) -> Response:
        openai_client: OpenAI = self.client.get_openai_client()
        options = {}
        if self.agent:
            options["extra_body"] = {
                "agent": {"name": self.agent.name, "type": "agent_reference"}
            }
        else:
            options['model'] = self.model

        response = openai_client.responses.create(
            input=[{"role": "user", "content": _} for _ in user_prompt],
            **options
        )
        return response



from typing import Any

from azure.core.credentials import TokenCredential as AzTokenCredential, AccessToken
from azure.identity._internal.get_token_mixin import GetTokenMixin

default_scopes = [
    'https://management.azure.com/.default'  # for Azure Resource Manager
]


class TokenCredential(AzTokenCredential):
    def __init__(self, credential: GetTokenMixin):
        self.credential = credential

    def get_token(
            self,
            *scopes: str,
            claims: str|None = None,
            tenant_id: str|None = None,
            enable_cae: bool = False,
            **kwargs: Any,
    ) -> AccessToken:
        if not scopes:
            scopes = default_scopes
        return self.credential.get_token(
            *scopes,
            claims=claims, tenant_id=tenant_id, enable_cae=enable_cae,
            **kwargs)

    def __getattr__(self, item):
        # Delegate unknown attributes/methods to the wrapped instance
        return getattr(self.credential, item)



from dataclasses import dataclass
from time import sleep
from typing import Iterable

from azure.core.exceptions import ResourceNotFoundError, ResourceExistsError, HttpResponseError
from azure.mgmt.monitor import MonitorManagementClient
from azure.mgmt.monitor.models import AzureMonitorWorkspaceResource, ProvisioningState
from azure.mgmt.monitor.operations import AzureMonitorWorkspacesOperations

from davidkhala.azure import TokenCredential
from davidkhala.azure.monitor import AbstractResource


class Management:
    def __init__(self, credential: TokenCredential, subscription_id: str):
        self.client = MonitorManagementClient(credential, subscription_id)

    @property
    def dce(self):
        return self.client.data_collection_endpoints

    @property
    def dcr(self):
        return self.client.data_collection_rules

    @property
    def workspace(self):
        return Workspace(self.client.azure_monitor_workspaces)


class Workspace:
    """
    Azure Monitor workspace
    """

    @dataclass
    class Resource(AbstractResource):
        data_collection_rule: str
        data_collection_endpoint: str
        state: ProvisioningState | str  # one of ["Creating", "Succeeded", "Deleting","Failed", and "Canceled"]

        def from_resource(self, resource: AzureMonitorWorkspaceResource):
            super().from_resource(resource)
            assert resource.default_ingestion_settings is not None, f"{resource} missing .default_ingestion_settings"
            self.data_collection_rule = resource.default_ingestion_settings.data_collection_rule_resource_id
            self.data_collection_endpoint = resource.default_ingestion_settings.data_collection_endpoint_resource_id
            self.state = resource.provisioning_state
            self.immutable_id = resource.account_id
            return self

        def default_dcr(self, client: MonitorManagementClient):
            """
            The managed DCR.
            Permission denied from any change attempt
            :param client:
            :return: DCR.Resource
            """
            from davidkhala.azure.monitor.dcr import DCR
            control = DCR(client.data_collection_rules)
            got = control.get_by_id(self.data_collection_rule)
            assert got.stream_declarations is None
            return got

    def __init__(self, azure_monitor_workspaces: AzureMonitorWorkspacesOperations):
        self.azure_monitor_workspaces = azure_monitor_workspaces

    def list(self) -> Iterable[AzureMonitorWorkspaceResource]:
        return self.azure_monitor_workspaces.list_by_subscription()

    def create(self, resource_group_name: str, name: str, location="East Asia") -> Resource:
        r = self.get(resource_group_name, name)
        if r:
            if r.state == ProvisioningState.SUCCEEDED:
                return r
            else:
                while r is not None:
                    assert r.state == ProvisioningState.DELETING, f"expected {ProvisioningState.DELETING}, but got {r.state}"
                    sleep(1)
                    r = self.get(resource_group_name, name)

        while True:

            try:
                r = self.azure_monitor_workspaces.create(
                    resource_group_name, name,
                    AzureMonitorWorkspaceResource(location=location)
                )
                break
            except ResourceExistsError as e:
                if e.message in [
                    "There is a Create operation currently running on this Monitoring account",
                    "Monitoring account has been modified before operation completed"
                ]:
                    sleep(1)
                else:
                    raise
            except HttpResponseError as e:
                if 'Code: InconsistentState' in str(e):
                    sleep(30)
                else:
                    raise

        wrapped = Workspace.Resource(*[None] * 8).from_resource(r)
        assert wrapped.state == ProvisioningState.SUCCEEDED
        return wrapped

    def get(self, resource_group_name: str, name: str) -> Resource | None:
        try:
            r = self.azure_monitor_workspaces.get(resource_group_name, name)
            return Workspace.Resource(*[None] * 8).from_resource(r)
        except ResourceNotFoundError:
            return None

    def delete_async(self, resource_group_name: str, name: str):
        return self.azure_monitor_workspaces.begin_delete(resource_group_name, name)

    def wait_until_gone(self, resource_group_name: str, name: str):
        r = self.get(resource_group_name, name)
        while r is not None:
            assert r.state == ProvisioningState.DELETING, f"expected {ProvisioningState.DELETING}, but got {r.state}"
            r = self.get(resource_group_name, name)

    def delete(self, resource_group_name: str, name: str):
        promise = self.delete_async(resource_group_name, name)
        assert promise.result() is None



from typing import Iterable

from azure.core.credentials import TokenCredential
from azure.mgmt.resource import SubscriptionClient
from azure.mgmt.resource.subscriptions.models import Subscription as NativeSubscription


class Subscription:
    def __init__(self, credential: TokenCredential):
        self.client = SubscriptionClient(credential)

    def list(self) -> Iterable[NativeSubscription]:
        return self.client.subscriptions.list()

    def get(self, subscription_id: str) -> NativeSubscription:
        return self.client.subscriptions.get(subscription_id)

    def get_one(self) -> NativeSubscription:
        return next(iter(self.list()))




from typing import Union, Iterable, AnyStr, IO

from azure.core.exceptions import ClientAuthenticationError
from azure.storage.blob import BlobServiceClient

from davidkhala.azure import TokenCredential


class Object:
    def __init__(self, service_client: BlobServiceClient, container: str, file_name: str):
        self.client = service_client.get_blob_client(container, file_name)

    def upload(self, data: Union[bytes, str, Iterable[AnyStr], IO[bytes]], *, overwrite=True):
        self.client.upload_blob(data, overwrite=overwrite)


class Client:
    def __init__(self, account_name: str, credential: TokenCredential | str):
        """

        :param account_name:
        :param credential: could be the access key in str
        """
        self.client = BlobServiceClient(
            account_url=f"https://{account_name}.blob.core.windows.net",
            credential=credential
        )

    def connect(self):
        try:
            self.account_information
            return True

        except ClientAuthenticationError as e:
            if (e.status_code == 401 and e.reason == 'Server failed to authenticate the request. Please refer to the information in the www-authenticate header.'):
                return False
            raise e

    def blob(self, container: str, file_name: str) -> Object:
        return Object(self.client, container, file_name)

    @property
    def account_information(self):
        data = self.client.get_account_information()
        return {
            'version': data.get('version'),
            'date': data.get('date'),
            'sku_name': data.get("sku_name"),
            'account_kind': data.get('account_kind'),
            'is_hns_enabled': data.get("is_hns_enabled"),
        }





def fromDataFrame(df, blob_path:str):
    import pandas as pd
    import pyarrow as pa
    import pyarrow.parquet as pq
    from adlfs import AzureBlobFileSystem

    # 初始化文件系统
    fs = AzureBlobFileSystem(
        account_name="your_account",
        account_key="your_key"
    )

    # DataFrame → Parquet buffer
    table = pa.Table.from_pandas(df)
    buffer = pa.BufferOutputStream()
    pq.write_table(table, buffer)

    # 上传到 Azure
    with fs.open(blob_path, "wb") as f:
        f.write(buffer.getvalue().to_pybytes())



import json
import subprocess
from typing import Iterator, Tuple, TypedDict

from azure.core.credentials import TokenProvider
from azure.identity import (
    DefaultAzureCredential, AzureCliCredential,
    EnvironmentCredential, ManagedIdentityCredential, SharedTokenCacheCredential,
    AzurePowerShellCredential, AzureDeveloperCliCredential, ClientSecretCredential,
)
from davidkhala.utils.syntax.env import is_windows, is_mac, is_linux

from davidkhala.azure import TokenCredential, default_scopes

DefaultCredentialType = EnvironmentCredential | ManagedIdentityCredential | SharedTokenCacheCredential | AzureCliCredential | AzurePowerShellCredential | AzureDeveloperCliCredential
default = DefaultAzureCredential


def from_service_principal(tenant_id: str, client_id: str, client_secret: str) -> TokenCredential:
    return TokenCredential(ClientSecretCredential(tenant_id, client_id, client_secret))


from azure.identity import CredentialUnavailableError


class CliCredential:
    class Structure(TypedDict):
        class User(TypedDict):
            name: str
            type: str

        class Tenant(TypedDict):
            tenantId: str

        environmentName: str
        homeTenantId: str
        id: str
        isDefault: bool
        managedByTenants: list[Tenant]
        name: str
        state: str
        tenantId: str
        user: User

    @staticmethod
    def current():
        command = None
        if is_windows():
            command = 'az.cmd'
        elif is_linux():
            command = '/usr/bin/az'
        elif is_mac():
            # TODO
            ...

        r = subprocess.run([command, 'account', 'show'], capture_output=True, text=True)
        data = r.stdout.strip()
        return CliCredential.Structure(**json.loads(data))


def actually(credentials: DefaultAzureCredential) -> Iterator[Tuple[TokenProvider, int]]:
    """
    Personal Microsoft account is not supported to get_token(...)
    :param credentials:
    :return:
    """
    for i, credential in enumerate(credentials.credentials):
        try:
            credential.get_token(*default_scopes)
            yield credential, i
        except CredentialUnavailableError as na_error:
            err_str = str(na_error)

            swallow: bool = False
            match i:
                case 0:
                    swallow = err_str == """EnvironmentCredential authentication unavailable. Environment variables are not fully configured.
Visit https://aka.ms/azsdk/python/identity/environmentcredential/troubleshoot to troubleshoot this issue."""
                case 1:
                    swallow = err_str in [
                        'ManagedIdentityCredential authentication unavailable, no response from the IMDS endpoint.',
                        'ManagedIdentityCredential authentication unavailable, no response from the IMDS endpoint. invalid_request'
                    ]
                case 2:
                    swallow = err_str == 'SharedTokenCacheCredential authentication unavailable. No accounts were found in the cache.'
                case 3:
                    swallow = err_str == "Please run 'az login' to set up an account"
                case 4:
                    swallow = err_str in [
                        'Az.Account module >= 2.2.0 is not installed',
                        'Please run "Connect-AzAccount" to set up account'
                    ]

                case 5:
                    swallow = err_str in [
                        "Azure Developer CLI could not be found. Please visit https://aka.ms/azure-dev for installation instructions and then,once installed, authenticate to your Azure account using 'azd auth login'.",
                        "Please run 'azd auth login' from a command prompt to authenticate before using this credential."
                    ]
            if not swallow:
                raise na_error



from typing import List

from azure.identity import ClientSecretCredential
from azure.mgmt.loganalytics.models import ColumnTypeEnum
from azure.monitor.ingestion import LogsIngestionClient

from davidkhala.azure import TokenCredential
from davidkhala.azure.monitor.dce import DCE
from davidkhala.azure.monitor.dcr import DCR


class Ingestion:
    logs: List[dict]
    stream_name: str
    dcr: DCR.Resource
    schema: dict[str, ColumnTypeEnum | str]

    def __init__(self, credential: TokenCredential, dcr: DCR.Resource, end_point: str | None,
                 *,
                 dce_operations: DCE = None):
        assert type(credential.credential) == ClientSecretCredential
        if end_point is None:
            end_point = dce_operations.get_by_id(dcr.data_collection_endpoint_id).logs_ingestion
        """
        The Data Collection Endpoint for the Data Collection Rule (e.g. https://dce-name.eastus-2.ingest.monitor.azure.com.)
        """
        self.dcr = dcr
        self.client = LogsIngestionClient(end_point, credential)

    def getLogger(self, stream_name: str = None):
        if stream_name is None:
            stream_name = self.dcr.get_one_stream()
        assert stream_name is not None
        self.stream_name = stream_name
        self.schema = self.dcr.schema(stream_name)
        del self.schema['TimeGenerated']
        self.logs = []
        return self

    def commit(self):
        self.client.upload(self.dcr.immutable_id, self.stream_name, self.logs)

    def log(self, message: dict):
        assert message.keys() == self.schema.keys()
        self.logs.append(message)



import time
from typing import Iterator

from azure.ai.agents import AgentsClient
from azure.ai.agents.models import Agent, ToolDefinition, AgentThreadCreationOptions, ThreadMessageOptions, ThreadRun, \
    RunStatus, ThreadMessage, MessageTextContent
from azure.core.credentials import TokenCredential


class Message:
    def __init__(self, _: ThreadMessage):
        self._ = _

    @property
    def text_messages(self) -> list[str]:
        return [content['text']['value'] for content in self._.content if isinstance(content, MessageTextContent)]


class Client:
    def __init__(self, foundry_id, project, credential: TokenCredential) -> None:
        self.agent: Agent | None = None
        self.client = AgentsClient(
            endpoint=f"https://{foundry_id}.services.ai.azure.com/api/projects/{project}",
            credential=credential
        )
        self.project = project

    def as_agent(self, model: str, *,
                 tools: list[ToolDefinition],
                 sys_prompt: str = None,
                 ):
        self.agent = self.client.create_agent(
            model=model,
            instructions=sys_prompt,
            tools=tools,
        )

    def begin_chat(self, *messages: ThreadMessageOptions) -> ThreadRun:
        thread = AgentThreadCreationOptions(
            messages=list(messages)
        )
        return self.client.create_thread_and_run(
            agent_id=self.agent.id,
            thread=thread,
            parallel_tool_calls=True,
        )

    def wait_for_run(self, thread_run: ThreadRun, expected_status: RunStatus, poll_interval=1):
        current_status = self.client.runs.get(thread_run.thread_id, thread_run.id).status

        if current_status == expected_status:
            return None
        time.sleep(poll_interval)
        return self.wait_for_run(thread_run, expected_status, poll_interval)

    def get_messages(self, thread_run: ThreadRun) -> Iterator[Message]:
        return (Message(m) for m in self.client.messages.list(thread_run.thread_id, run_id=thread_run.id))



from dataclasses import dataclass
from typing import Dict, List, Iterable

from azure.mgmt.monitor.models import (
    StreamDeclaration,
    DataCollectionRuleResource, DataCollectionRuleDestinations, DataCollectionRuleDataSources,
    KnownColumnDefinitionType, ColumnDefinition,
    LogAnalyticsDestination, DataFlow,
)
from azure.mgmt.monitor.operations import DataCollectionRulesOperations

from davidkhala.azure.monitor import AbstractResource


class DCR:
    @dataclass
    class Resource(AbstractResource):
        stream_declarations: Dict[str, StreamDeclaration] | None
        data_collection_endpoint_id: str

        def from_resource(self, resource: DataCollectionRuleResource):
            super().from_resource(resource)
            self.stream_declarations = resource.stream_declarations
            self.data_collection_endpoint_id = resource.data_collection_endpoint_id
            return self

        def get_one_stream(self) -> str | None:
            if self.stream_declarations is not None:
                for stream in self.stream_declarations.keys():
                    return stream

        def schema(self, stream_name) -> dict[str, KnownColumnDefinitionType | str]:
            r = {}
            for column in self.stream_declarations[stream_name].columns:
                r[column.name] = column.type
            return r

    class Destinations:
        def __init__(self, destinations: DataCollectionRuleDestinations):
            mapper = lambda l: [] if l is None else (item.name for item in l)

            self.names = [
                *mapper(destinations.log_analytics),
                *mapper(destinations.monitoring_accounts),
                *mapper(destinations.azure_monitor_metrics),
                *mapper(destinations.event_hubs),
                *mapper(destinations.event_hubs_direct),
                *mapper(destinations.storage_blobs_direct),
                *mapper(destinations.storage_tables_direct),
                *mapper(destinations.storage_accounts),
            ]

    def __init__(self, data_collection_rules: DataCollectionRulesOperations):
        self.data_collection_rules = data_collection_rules

    def get(self, resource_group_name: str, name: str) -> Resource:
        r = self.data_collection_rules.get(resource_group_name, name)
        return DCR.Resource(*[None] * 7).from_resource(r)

    def get_by_id(self, resource_id: str):
        words = resource_id.split("/")
        return self.get(words[4], words[-1])

    def list(self) -> Iterable[DataCollectionRuleResource]:
        return self.data_collection_rules.list_by_subscription()

    def delete(self, resource_group_name: str, name: str):
        return self.data_collection_rules.delete(resource_group_name, name)


from davidkhala.azure.monitor.log import AnalyticsWorkspace
from davidkhala.azure.monitor.dce import DCE


class Factory:
    location: str
    data_collection_endpoint_id: str

    def __init__(self,
                 resource_group_name: str,
                 name: str):

        self.resource_group_name = resource_group_name
        self.name = name
        self.stream_declarations = {}
        self.destinations = DataCollectionRuleDestinations()
        self.data_flows: List[DataFlow] = []
        self.data_sources = DataCollectionRuleDataSources()

    def with_DataCollectionEndpoint(self, dce: DCE.Resource):
        self.location = dce.location
        self.data_collection_endpoint_id = dce.id
        return self

    def with_LogAnalyticsTable(self, name, schema: dict[str, KnownColumnDefinitionType | str],
                               workspace: AnalyticsWorkspace.Resource):
        stream_name = f"Custom-{name}_CL"
        schema["TimeGenerated"] = KnownColumnDefinitionType.DATETIME  # decorate
        columns = [ColumnDefinition(name=name, type=_type) for name, _type in schema.items()]
        self.stream_declarations[stream_name] = StreamDeclaration(columns=columns)

        _workspace = LogAnalyticsDestination(
            workspace_resource_id=workspace.id,
            name=workspace.immutable_id.replace('-', ''),
        )
        _workspace.workspace_id = workspace.immutable_id

        if self.destinations.log_analytics is None:
            self.destinations.log_analytics = [_workspace]
        else:
            self.destinations.log_analytics.append(_workspace)

        self.data_flows.append(DataFlow(
            streams=[stream_name],
            destinations=[_workspace.name],
            transform_kql="source | extend TimeGenerated = now()",
            output_stream=stream_name
        ))
        return self

    def build(self, operations: DataCollectionRulesOperations):
        body = DataCollectionRuleResource(
            location=self.location,
            data_collection_endpoint_id=self.data_collection_endpoint_id,
            stream_declarations=self.stream_declarations,
            data_sources=self.data_sources,
            destinations=self.destinations,
            data_flows=self.data_flows,
        )
        return operations.create(self.resource_group_name, self.name, body)



from typing import Callable, TypedDict

from azure.cli.core import AzCommandsLoader
from azure.cli.core.commands import AzCliCommand

class HandlerParam(TypedDict):
    cmd:AzCliCommand

Handler = Callable[[HandlerParam], None]


class COMMAND_LOADER_CLS(AzCommandsLoader):

    def set_command(self, name:str, hanlder:Handler):
        self.command_table[name] = AzCliCommand(
            self, '-', hanlder,
        )



from dataclasses import dataclass


@dataclass
class AbstractResource:
    immutable_id: str | None
    id: str
    location: str
    name: str
    resource_group_name: str

    def from_resource(self, resource):
        self.id = resource.id
        self.location = resource.location
        self.resource_group_name = resource.id.split('/')[4]
        self.name = resource.name
        if hasattr(resource, 'immutable_id'):
            self.immutable_id = resource.immutable_id



from pathlib import Path
from typing import Any

from azure.ai.documentintelligence import DocumentIntelligenceClient
from azure.ai.documentintelligence.models import AnalyzeDocumentRequest, DocumentAnalysisFeature, AnalyzeResult


class Recognizer:
    def __init__(self, endpoint, credential):
        self.client = DocumentIntelligenceClient(endpoint=endpoint, credential=credential)

    def extract_entities(self, file: Path, focus: list[str] = None)->list[dict]:
        options:dict[str, Any] = {
            'model_id': "prebuilt-layout",
            'features': [DocumentAnalysisFeature.KEY_VALUE_PAIRS],
        }
        if focus:
            options['features'].append(DocumentAnalysisFeature.QUERY_FIELDS)
            options['query_fields'] = focus

        r = self.process(file, **options)
        if focus:
            return [
                {_: v.content for _, v in doc.fields.items()}
                for doc in r.documents
            ]

        return [
            {pair['key']['content']: pair['value']['content'] if pair.get('value') else None}
            for pair in r.key_value_pairs
        ]

    @staticmethod
    def dict_of(result: AnalyzeResult):
        d = result.as_dict()
        del d['apiVersion']
        del d['modelId']
        del d['stringIndexType']
        return d

    def process(self, file: Path, **kwargs) -> AnalyzeResult:
        with open(file, "rb") as f:
            content = f.read()
        promise = self.client.begin_analyze_document(
            body=AnalyzeDocumentRequest(bytes_source=content),
            **kwargs
        )
        return promise.result()



# https://github.com/fsspec/adlfs
from adlfs import AzureBlobFileSystem, AzureBlobFile
from azure.core.exceptions import ClientAuthenticationError


class FS:
    def __init__(self, account_name: str, account_key: str):
        self._ = AzureBlobFileSystem(
            account_name=account_name,
            account_key=account_key,
        )

    def connect(self):
        try:
            containers = self._.ls("")
            assert isinstance(containers, list)
            return True
        except ClientAuthenticationError as e:
            if (
                    e.status_code == 403 and
                    e.reason == 'Server failed to authenticate the request. Make sure the value of Authorization header is formed correctly including the signature.'
            ): return False
            raise e

class File(AzureBlobFile):

    def __enter__(self):
        self.connect_client()
        return super().__enter__()


from dataclasses import dataclass
from typing import Iterable

from azure.mgmt.monitor.models import (
    DataCollectionEndpointResource, DataCollectionEndpointNetworkAcls, KnownPublicNetworkAccessOptions
)
from azure.mgmt.monitor.operations import DataCollectionEndpointsOperations

from davidkhala.azure.monitor import AbstractResource


class DCE:
    def __init__(self, data_collection_endpoints: DataCollectionEndpointsOperations):
        self.data_collection_endpoints = data_collection_endpoints

    @dataclass
    class Resource(AbstractResource):
        configuration_access: str
        logs_ingestion: str

        def from_resource(self, r: DataCollectionEndpointResource):
            super().from_resource(r)
            self.configuration_access = r.configuration_access.endpoint
            self.logs_ingestion = r.logs_ingestion.endpoint
            return self

    def create(self, resource_group_name, name, location="East Asia") -> Resource:
        body = DataCollectionEndpointResource(
            location=location,
            network_acls=DataCollectionEndpointNetworkAcls(
                public_network_access=KnownPublicNetworkAccessOptions.ENABLED
            )
        )
        return DCE.Resource(*[None] * 7).from_resource(
            self.data_collection_endpoints.create(resource_group_name, name, body)
        )

    def get(self, resource_group_name, name) -> Resource:
        return DCE.Resource(*[None] * 7).from_resource(
            self.data_collection_endpoints.get(resource_group_name, name)
        )

    def get_by_id(self, resource_id: str):
        words = resource_id.split("/")
        return self.get(words[4], words[-1])

    def delete(self, resource_group_name: str, name: str):
        self.data_collection_endpoints.delete(resource_group_name, name)

    def list(self) -> Iterable[DataCollectionEndpointResource]:
        return self.data_collection_endpoints.list_by_subscription()
